namespace GossamerVR.MorphFilterPlus
{
    using System;
    using System.Collections.Generic;
    using UnityEngine;
    using UnityEngine.UI;

    /// <summary>
    /// replaces or covers an input field with a copy,
    /// which can intercept or forward events to the original.
    /// </summary>
    public class InputFieldBridge: UIElementBridge<InputField>
    {
        public bool IsTagField { get; private set; }
        
        public InputFieldBridge(InputField oldElement, bool isTagField, int panel, int gender, bool hideOldElement, bool forwardEvents) : base(oldElement, panel, gender, hideOldElement, forwardEvents) 
        {
            this.IsTagField = isTagField;
        }
        
        protected override void InitNewElement()
        {
            this.NewElement.onValueChanged.RemoveAllListeners();
            this.NewElement.onEndEdit.RemoveAllListeners();            
            this.NewElement.onValueChanged.AddListener(this.OnInputChange);
            this.NewElement.interactable = true;

            // the clear button is duplicated too as a child, if it has one.
            var clearButton = this.NewElement.GetComponentInChildren<Button>();
            if (clearButton != null)
            {
                clearButton.onClick.RemoveAllListeners();
                clearButton.onClick.AddListener(this.OnClear);
            }

            // add our signature MorphFilterBlue(TM)
            this.NewElement.GetComponent<Image>().color = MorphFilterPlus.MintyFreshColor;

                //// we also need to know when the filter gets enabled so we can re-apply it,
                //// since the UI refreshes every time it's opened. this is a little inelegant imo
                //// but it will do the trick for now. since it's part of the field
                //// and the field is destroyed on reversion, we don't need to worry about lifecycle.
                //var listener = this.NewElement.gameObject.AddComponent<StateListener>();
                //listener.Owner = person;
                //listener.EnabledEvent += this.OnMorphUIEnabled;
        }

        private void OnInputChange(string value)
        {
            if (this.canInteract)
            {
                this.Interact();
                if (this.forwardEvents && this.OldElement != null)
                {
                    this.OldElement.text = value;
                }
            }
        }

        private void OnClear()
        {
            this.NewElement.text = "";
        }

        protected override void SetupOverlay()
        {
            SuperController.LogError("I had not prepared for an input field to be an overlay. Why am I calling this?");
        }

        protected override void OnRemove()
        {
            
            this.NewElement.text = "";
            this.OldElement.text = "";

        }
    }
}