namespace GossamerVR.MorphFilterPlus
{
    using System;
    using System.Collections.Generic;
    using UnityEngine;
    using UnityEngine.UI;

    /// <summary>
    /// replaces or covers a button with a copy,
    /// which can intercept or forward events to the original.
    /// </summary>
    public class ToggleBridge : UIElementBridge<Toggle>
    {
        private bool initialValue;
        public ToggleBridge(Toggle oldElement, int panel, int gender, bool initialValue, bool hideOldElement, bool forwardEvents) : base(oldElement, panel, gender,  hideOldElement, forwardEvents) 
        {
            this.NewElement.isOn = initialValue;
        }
        

        protected override void InitNewElement()
        {            
            this.NewElement.onValueChanged.RemoveAllListeners();
            this.NewElement.onValueChanged.AddListener(this.OnToggle);
            this.NewElement.interactable = true;
        }

        protected override void SetupOverlay()
        {
            //foreach(var image in this.NewElement.GetComponentsInChildren<Image>())
            //{
            //    image.color = Color.clear;
            //}            
            foreach (var label in this.NewElement.GetComponentsInChildren<Text>())
            {
                label.text = "        ";
            }
        }

        /// <summary>
        /// The action to perform when clicking the toggle. subscribed in InitNewElement()
        /// </summary>
        private void OnToggle(bool value)
        {            
            if (this.canInteract)
            {
                this.Interact();
                if (this.forwardEvents && this.OldElement != null)
                {
                    this.OldElement.isOn = this.NewElement.isOn;
                }
            }

        }

        protected override void OnRemove() {}

    }
}
