namespace GossamerVR
{
    using System;
    using System.Collections.Generic;
    using UnityEngine;

    /// <summary>
    /// A class made to find all the bits and pieces of a given person atom.
    /// </summary>
    public static class PersonUtil
    {

        /// <summary>
        /// The geometry transform relative to the person atom.
        /// </summary>
        public const string GeomPath = "rescale2/geometry";
        

        public static class Joint
        {
            public const string None = "None";
            public const string Root = "Root";
            public const string Head = "Head";
            public const string Neck = "Neck";
            public const string Chest = "Chest";
            public const string Abdomen = "Abdomen";
            public const string Pelvis = "Pelvis";
            public const string Hip = "Hip";

            public const string RShoulder = "R Shoulder";
            public const string RUpperArm = "R Upper Arm";
            public const string RForearm = "R Forearm";
            public const string RHand = "R Hand";
            public const string RThumb = "R Thumb";
            public const string RIndexF = "R Index F";
            public const string RMiddleF = "R Middle F";
            public const string RRingF = "R Ring F";
            public const string RPinkyF = "R Pinky F";

            public const string LShoulder = "L Shoulder";
            public const string LUpperArm = "L Upper Arm";
            public const string LForearm = "L Forearm";
            public const string LHand = "L Hand";
            public const string LThumb = "L Thumb";
            public const string LIndexF = "L Index F";
            public const string LMiddleF = "L Middle F";
            public const string LRingF = "L Ring F";
            public const string LPinkyF = "L Pinky F";

            public const string RThigh = "R Thigh";
            public const string RShin = "R Shin";
            public const string RFoot = "R Foot";

            public const string LThigh = "L Thigh";
            public const string LShin = "L Shin";
            public const string LFoot = "L Foot";

        }

        public static class JointPath
        {
            // the root path is from a person's atom transform.
            public const string Root = "rescale2/PhysicsModel/Genesis2Female";

            // all other paths are relative to the root
            public const string Hip = "hip";
            public const string Pelvis = Hip + "/pelvis";
            public const string Abdomen = Hip + "/abdomen";
            public const string Chest = Abdomen + "/abdomen2/chest";
            public const string Neck = Chest + "/neck";
            public const string Head = Neck + "/head";

            public const string RShoulder = Chest + "/rCollar";
            public const string RUpperArm = RShoulder + "/rShldr";
            public const string RForearm = RUpperArm + "/rForeArm";
            public const string RHand = RForearm + "/rHand";
            public const string RThumb = RHand + "/rThumb1";
            public const string RIndexF = RHand + "/rCarpal1/rIndex1";
            public const string RMiddleF = RHand + "/rCarpal1/rMid1";
            public const string RRingF = RHand + "/rCarpal2/rRing1";
            public const string RPinkyF = RHand + "/rCarpal2/rPinky1";

            public const string LShoulder = Chest + "/lCollar";
            public const string LUpperArm = LShoulder + "/lShldr";
            public const string LForearm = LUpperArm + "/lForeArm";
            public const string LHand = LForearm + "/lHand";
            public const string LThumb = LHand + "/lThumb1";
            public const string LIndexF = LHand + "/lCarpal1/lIndex1";
            public const string LMiddleF = LHand + "/lCarpal1/lMid1";
            public const string LRingF = LHand + "/lCarpal2/lRing1";
            public const string LPinkyF = LHand + "/lCarpal2/lPinky1";

            public const string RThigh = Hip + "/pelvis/rThigh";
            public const string RShin = RThigh + "/rShin";
            public const string RFoot = RShin + "/rFoot";

            public const string LThigh = Hip + "/pelvis/lThigh";
            public const string LShin = LThigh + "/lShin";
            public const string LFoot = LShin + "/lFoot";

        }

        public static class UIPath
        {
            public const string Root = "UI/UIPlaceHolderModel/UIModel";
            public const string Content = Root + "/Canvas/Panel/Content";
            
            // the names of the panels for each ui tab change
            // once the character is first selected, so both are tracked.
            public const string MorphFemale = Content + "/Female Morphs";
            public const string MorphFemaleAlt = Content + "/TabMorphsFemale";

            public const string MorphMale = Content + "/Male Morphs";
            public const string MorphMaleAlt = Content + "/TabMorphsMale";

            public const string Cloth = Content + "/Clothing";
            public const string ClothAlt = Content + "/TabClothing";

            public const string Hair = Content + "/Hair";
            public const string HairAlt = Content + "/TabHair";


        }


        public static class MorphPath
        {
            public const string BankRoot = PersonUtil.GeomPath + "/MorphBanks";
            /// <summary>
            /// the array of morph banks collectively storing all the character morphs.
            /// we store them as an array so we can foreach() them easily. Their order
            /// is important because we'll use the MorphBankPath enum as indices.
            /// </summary>
            public static readonly string[] Banks = new[]
            {
                MorphPath.BankRoot + "/FemaleCharacterMorphBank(Clone)",
                MorphPath.BankRoot + "/FemaleGenMorphBank(Clone)",
                MorphPath.BankRoot + "/MaleCharacterMorphBank(Clone)",
                MorphPath.BankRoot + "/MaleGenMorphBank(Clone)",
                MorphPath.BankRoot + "/MaleGen2MorphBank(Clone)",
            };
            

        }
        


        private static Dictionary<string, string> PathTable = new Dictionary<string, string>()
        {
            {Joint.Root, JointPath.Root},
            {Joint.Hip, JointPath.Hip },
            {Joint.Abdomen, JointPath.Abdomen },
            {Joint.Pelvis, JointPath.Pelvis },
            {Joint.Chest, JointPath.Chest },
            {Joint.Neck, JointPath.Neck },
            {Joint.Head, JointPath.Head },

            {Joint.RShoulder, JointPath.RShoulder },
            {Joint.RUpperArm, JointPath.RUpperArm},
            {Joint.RForearm, JointPath.RForearm },
            {Joint.RHand, JointPath.RHand },
            {Joint.RThumb, JointPath.RThumb },
            {Joint.RIndexF, JointPath.RIndexF },
            {Joint.RMiddleF, JointPath.RMiddleF },
            {Joint.RRingF, JointPath.RRingF },
            {Joint.RPinkyF, JointPath.RPinkyF },

            {Joint.LShoulder, JointPath.LShoulder },
            {Joint.LUpperArm, JointPath.LUpperArm},
            {Joint.LForearm, JointPath.LForearm },
            {Joint.LHand, JointPath.LHand },
            {Joint.LThumb, JointPath.LThumb },
            {Joint.LIndexF, JointPath.LIndexF },
            {Joint.LMiddleF, JointPath.LMiddleF },
            {Joint.LRingF, JointPath.LRingF },
            {Joint.LPinkyF, JointPath.LPinkyF },

            {Joint.RThigh, JointPath.RThigh },
            {Joint.RShin, JointPath.RShin},
            {Joint.RFoot, JointPath.RFoot },

            {Joint.LThigh, JointPath.LThigh },
            {Joint.LShin, JointPath.LShin },
            {Joint.LFoot, JointPath.LFoot },
        };

        public static string GetPath(string joint)
        {
            string ret_path = "";

            if (!PathTable.TryGetValue(joint, out ret_path))
            {
                SuperController.LogError($"PersonUtil.GetPath() has no path for joint: '{joint}'");
            }            
            return ret_path;
        }
        
        public static Transform FindRoot(Atom person)
        {
            Transform ret_root = null;
            string path = GetPath(Joint.Root);
            if (person != null && !string.IsNullOrEmpty(path))
            {
                ret_root =person.transform.Find(path);
            }
            return ret_root;
        }

        public static Transform FindJoint(Transform root, string joint)
        {
            Transform ret_joint = null;
            string path = GetPath(joint);
            if (root != null && !string.IsNullOrEmpty(path))
            {
                ret_joint = root.Find(path);
            }
            return ret_joint;
        }

    }
}